@@ -25,22 +25,13 @@ struct text_file_section {
 
 #define LOAD_TEXT_FILE_REMOVE_QUOTES 0x00000001
 #define LOAD_TEXT_FILE_NO_WARNINGS   0x00000002
+#define LOAD_TEXT_FILE_ALLOW_STDIN   0x00000004
 
 extern int
-do_load_text_file(const tchar *path,
-		  const void *buf, size_t bufsize, void **mem_ret,
-		  const struct text_file_section *pos_sections,
-		  int num_pos_sections, int flags,
-		  line_mangle_t mangle_line);
-
-static inline int
-load_text_file(const tchar *path, void **mem_ret,
+load_text_file(const tchar *path, const void *buf, size_t bufsize,
+	       void **mem_ret,
 	       const struct text_file_section *pos_sections,
-	       int num_pos_sections, line_mangle_t mangle_line)
-{
-	return do_load_text_file(path, NULL, 0, mem_ret,
-				 pos_sections, num_pos_sections,
-				 LOAD_TEXT_FILE_REMOVE_QUOTES, mangle_line);
-}
+	       int num_pos_sections,
+	       int flags, line_mangle_t mangle_line);
 
 #endif /* _WIMLIB_TEXTFILE_H_ */
