@@ -30,7 +30,9 @@ typedef struct _RTL_RELATIVE_NAME_U {
 	PRTLP_CURDIR_REF CurDirRef;
 } RTL_RELATIVE_NAME_U, *PRTL_RELATIVE_NAME_U;
 
-#define FSCTL_SET_PERSISTENT_VOLUME_STATE 0x90238
+#ifndef FSCTL_SET_PERSISTENT_VOLUME_STATE
+#define FSCTL_SET_PERSISTENT_VOLUME_STATE \
+	CTL_CODE(FILE_DEVICE_FILE_SYSTEM, 142, METHOD_BUFFERED, FILE_ANY_ACCESS)
 
 #define PERSISTENT_VOLUME_STATE_SHORT_NAME_CREATION_DISABLED 0x00000001
 
@@ -40,6 +42,7 @@ typedef struct _FILE_FS_PERSISTENT_VOLUME_INFORMATION {
 	ULONG Version;
 	ULONG Reserved;
 } FILE_FS_PERSISTENT_VOLUME_INFORMATION, *PFILE_FS_PERSISTENT_VOLUME_INFORMATION;
+#endif /* FSCTL_SET_PERSISTENT_VOLUME_STATE */
 
 /* ntdll functions  */
 
