@@ -742,9 +742,12 @@ get_capture_config(const tchar *config_file, struct capture_config *config,
 
 		size_t len = tstrlen(fs_source_path) +
 			     tstrlen(wimboot_cfgfile);
-		tmp_config_file = MALLOC((len + 1) * sizeof(tchar));
 		struct stat st;
 
+		tmp_config_file = MALLOC((len + 1) * sizeof(tchar));
+		if (!tmp_config_file)
+			return WIMLIB_ERR_NOMEM;
+
 		tsprintf(tmp_config_file, T("%"TS"%"TS),
 			 fs_source_path, wimboot_cfgfile);
 		if (!tstat(tmp_config_file, &st)) {
